/*
   Pointers are positive integers but there
   is a difference between the arithmetic of
   int's and pointer arithmetic.
*/
#include <stdio.h>

int main()
{
   // This union lets us work with the variable z as
   // either a pointer or as an integer.
   union { int x;
           int* y;} z;

   int i = 137;
   int j = 296;

   // let z, as a pointer, point to i
   z.y = &i;

   // Use pointer arithmetic.
   // Notice by looking at the output that the address decreases
   // by 4 since an integer is 4 bytes and this example uses pointer
   // arithmetic.
   printf("z.y is %d, it points to %d\n", z.y, *z.y);
   printf("(z.y)-1 is %d, it points to %d\n", z.y-1, *(z.y-1));

   // print a blank line
   printf("\n");

   // Use integer arithmetic.
   // Notice by looking at the output that the address decreases
   // by 1 since this example uses regular integer arithmetic.
   printf("z.x is %d, it points to %d\n", z.x, *(int*)z.x);
   printf("(z.x)-1 is %d, it points to %d\n", z.x-1, *(int*)(z.x-1));
   // Notice that in the last printf, the dereferencing operation
   // accesses an integer that is "out of alignment", that is, the
   // address of the integer is not on a 32 bit word boundary.

   return 0;
}//main
