/*
   This program demonstrates how a static local variable works in a
   function. The static variable is not an automatic local variable,
   instead it is part of the data segment in the program's image.

   The static variable is initialized as part of the loading of the
   program. It is not initialized with every function call. The static
   variable keeps its value from one function call to another.

   You can see this by looking at the assembly language listing of this
   program and also by dumping this program's obj or exe file. You can
   also step through this program in its assembly language version using
   the Visual Studio debugger and see the static variable keeps its value
   between function calls.

   Notice that both local and global static variables are stored in the
   same place, in the data segment. What is it that makes a local static
   variable different from a global static variable? If the local static
   variable is not on the stack, what makes it local?
*/
#include <stdio.h>

static int u = 12; // global static variable

int myFunction(int x)
{
   // local static variable
   // only initialized once!
   static int y = 14;

   printf("Location of local static variable:  %#x\n", (int)(&y));

   int z = 2;
   y = z + x;
   return y;
}//myFunction

int main(int argc, char *argv[])
{
   printf("Location of global static variable: %#x\n", (int)(&u));

   myFunction(u);
   myFunction(u+1);
   return 0;
}//main
