/*
   There are four ways to create an array of structs.

   One of the four is equivalent to Java's way of creating an array of objets.

   There are two "kinds" of array of structs
      (a) an array of struct objects
      (b) an array of struct pointers
   And each of these two kinds of array can be allocated two ways,
      (i) automaticall (on the stack)
     (ii) dynamically (in the heap)
   so ther are four kinds of "array of struct".
*/
#include <stdio.h>
#include <stdlib.h>

struct MyType {
   int w;
   int x;
   int y;
   int z;
};

void f(struct MyType x[])
{
   int y0 = x[0].y;
   int y1 = x[1].y;
   int y2 = x[2].y;
   printf("in f, x[0].y = %d\n", y0);
   printf("in f, x[1].y = %d\n", y1);
   printf("in f, x[2].y = %d\n", y2);
}


void g(struct MyType *x[])  // void g(struct MyType **x)
{
   int y0 = x[0]->y;
   int y1 = x[1]->y;
   int y2 = x[2]->y;
   printf("in g, x[0]->y = %d\n", y0);
   printf("in g, x[1]->y = %d\n", y1);
   printf("in g, x[2]->y = %d\n", y2);
}


void h(struct MyType **x)  // void h(struct MyType *x[])
{
   int y0 = x[0]->y;
   int y1 = x[1]->y;
   int y2 = x[2]->y;
   printf("in h, x[0]->y = %d\n", y0);
   printf("in h, x[1]->y = %d\n", y1);
   printf("in h, x[2]->y = %d\n", y2);
}


int main(void)
{
   // an array of struct objects allocated automatically
   struct MyType array1[3] = {{11,12,13,14}, {21,22,23,24}, {31,32,33,34}};

   // an array of struct objects allocated dynamically
   struct MyType *array2 = (struct MyType*)malloc(3 * sizeof(struct MyType));
   array2[0].y = 43;
   array2[1].y = 53;
   array2[2].y = 63;

   // an array of struct pointers allocated automatically
   struct MyType *array3[3];
   array3[0] = (struct MyType*)malloc(sizeof(struct MyType));
   array3[1] = (struct MyType*)malloc(sizeof(struct MyType));
   array3[2] = (struct MyType*)malloc(sizeof(struct MyType));
   array3[0]->y = 73;
   array3[1]->y = 83;
   array3[2]->y = 93;

   // an array of struct pointers allocated dynamically
   struct MyType **array4 = (struct MyType**)malloc(3 * sizeof(struct MyType*));
   array4[0] = (struct MyType*)malloc(sizeof(struct MyType));
   array4[1] = (struct MyType*)malloc(sizeof(struct MyType));
   array4[2] = (struct MyType*)malloc(sizeof(struct MyType));
   array4[0]->y = 103;
   array4[1]->y = 203;
   array4[2]->y = 303;

   f(array1);
   f(array2);
   g(array3);
   g(array4);
   h(array3);
   h(array4);

   free( array4[2] );  // free the objects in array4
   free( array4[1] );
   free( array4[0] );
   free( array4 );     // free array4

   free( array3[2] );  // free the objects in array3
   free( array3[1] );
   free( array3[0] );

   free( array2 );     // free array2 (and its objects)

   return 0;
}
