/*
   All parameters in C are call-by-value, so
   the value of a caller's actual parameter is
   COPIED into the callee's formal parameter.

   Call-by-value of a pointer (to a pointer) value.
   Notice how the caller can re-allocate the callee's
   value in the heap!
*/
#include <stdio.h>
#include <stdlib.h>

void f(int **x) // call-by-value (of a pointer to a pointer)
{
   printf("in f, **x = %d\n", **x);
   // What do the following three lines do?
   free(*x);
   *x = (int*)malloc(sizeof(int));
   **x = 0;
}


int main(void)
{
   int *x;                         // Create a pointer (on the stack).
   x = (int*)malloc(sizeof(int));  // Create the pointee (in the heap).
   *x = 5;                         // Give the pointee a value.
   f(&x);                          // Create a pointer-to-pointer (notice the small change)
   printf("in main, *x = %d\n", *x);
   free(x); // De-allocate the pointee from the heap.

   return 0;
}
