/*
   All parameters in C are call-by-value, so
   the value of a caller's actual parameter is
   COPIED into the callee's formal parameter.

   Call-by-value of a pointer (to a pointer) value.
   Notice how the caller can re-allocate the callee's
   value in the heap!
*/
#include <stdio.h>
#include <stdlib.h>

struct MyType {
   int w;
   int x;
   int y;
   int z;
};

void f(struct MyType **x) // call-by-value (of a pointer to a pointer)
{
   printf("in f, (*(*x)).x = %d\n", (*(*x)).x);
   // What do the following six lines do?
   free(*x);
   *x = (struct MyType*)malloc(sizeof(struct MyType));
   (*x)->w = 1;
   (*x)->x = 0;
   (*x)->y = 3;
   (*x)->z = 4;
}


int main(void)
{
   struct MyType *x;                                  // Create a pointer (on the stack).
   x = (struct MyType*)malloc(sizeof(struct MyType)); // Create the pointee (in the heap).
   x->w = 1;                                          // Give the pointee a value.
   x->x = 2;
   x->y = 3;
   x->z = 4;
   f(&x);    // Create a pointer-to-pointer (notice the small change)
   printf("in main, x->x = %d\n", x->x);
   free(x); // De-allocate the pointee from the heap.

   return 0;
}
