/*
   All parameters in C are call-by-value, so
   the value of a caller's actual parameter is
   COPIED into the callee's formal parameter.

   Call-by-value of a pointer (to a pointer) value.
   This is similar to the last example but the caller's
   struct is allocated on stack, not in the heap. What
   happens when f() tries to de-allocate the struct?
*/
#include <stdio.h>
#include <stdlib.h>

struct MyType {
   int w;
   int x;
   int y;
   int z;
};

void f(struct MyType **x) // call-by-value (of a pointer to a pointer)
{
   printf("in f, (*(*x)).x = %d\n", (*(*x)).x);
   // What do the following lines do?
   free(*x); // How can f() know if it is, or is not, OK to call free()?
   *x = (struct MyType*)malloc(sizeof(struct MyType));
   (*x)->w = 1;
   (*x)->x = 0;
   (*x)->y = 3;
   (*x)->z = 4;
}


int main(void)
{
   struct MyType x;        // Create a struct on the stack.
   x.w = 1;                // Initialize it.
   x.x = 2;
   x.y = 3;
   x.z = 4;
   struct MyType *w = &x;  // Create a pointer to the struct.
   f(&w);                  // Create a pointer-to-pointer.
   printf("in main, w->x = %d\n", w->x);

   return 0;
}
