/*
   All parameters in C are call-by-value, so
   the value of a caller's actual parameter is
   COPIED into the callee's formal parameter.

   Call-by-value of a pointer value.
   Pointee is allocated in the heap.
*/
#include <stdio.h>
#include <stdlib.h>

struct MyType {
   int w;
   int x;
   int y;
   int z;
};

void f(struct MyType *x) // call-by-value (of a pointer)
{
   printf("in f, x->x = %d\n", (*x).x);
   x->x = 0;  // This doesn't change the caller's value of the pointer,
              // but it does change the caller's value of the pointee.

   x = (void*)x + 4;  // What does the "+ 4" mean?
   x->x = 0;          // What does this change? Why?
   x = (struct MyType*)((int*)x - 2); // What does the "- 2" mean?
   x->x = 0;
}


int main(void)
{
   struct MyType *x;                                  // Create the pointer (on the stack).
   x = (struct MyType*)malloc(sizeof(struct MyType)); // Create the pointee (in the heap).
   x->w = 1;                                          // Give the pointee a value.
   x->x = 2;
   x->y = 3;
   x->z = 4;
   f(x);
   printf("in main, x->x = %d\n", x->x);
   free(x); // De-allocate the pointee from the heap.

   return 0;
}
